# RTI.R - N Maizlish - January 16, 2020
# This program reads FARS 2016 csv data files (ACCIDENT< PERSON< VEHICLE)
# and aggregates fatal injuries for striking vehicle-victim vehicle matrix 
# by mode ftp://ftp.nhtsa.dot.gov/fars/)
# For alternative years, rename ACCIDENT, PERSON, VEHICLE files with year
# FILE + YEAR (e.g. 2018): ACCIDENT2018.csv, PERSON2018.csv, VEHICLE2018.csv

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Initialize Progrram
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# turn off scientific notation in data frames
options(scipen=999)
fars <- "C:/Users/Noneof/Documents/HealthLinks/ITHIM_MPO/USA/RTI/FARS"
setwd(fars)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Read FARS Person file and Subset for fatal injuries
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

person <- read.csv("PERSON2016.csv", header = TRUE, stringsAsFactors = F)  
person$victim_id <- paste0(person$ST_CASE,person$VEH_NO, person$PER_NO)
# person$party_id  <- paste0(person$ST_CASE,person$VEH_NO)
person           <- person[person$INJ_SEV==4,c("victim_id", "ST_CASE","VEH_NO",
                          "PER_NO","PER_TYP","INJ_SEV", "VE_FORMS")]

# Create victim mode for pedestrians and cyclists (9 = missing)
person$VMODE <- NA
person[person$VEH_NO==0 & person$PER_TYP==5, "VMODE"] <- 1
person[person$VEH_NO==0 & person$PER_TYP==6, "VMODE"] <- 2

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Read FARS Vehicle file to get parties to the fatal injury collision
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

vehicles <- read.csv("VEHICLE2016.csv", header = TRUE, stringsAsFactors = F)
vehcodes <- read.csv("BodyTypeCodes.csv", header = TRUE, stringsAsFactors = F)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Merge with Vehicle codes list to get ITHIM mode for motorized vehicles
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
vehicles <- merge(vehicles, vehcodes, by="BODY_TYP", all.x = TRUE) 

# vehicles$party_id <- paste0(vehicles$ST_CASE,vehicles$VEH_NO)
vehicles <- vehicles[,c("ST_CASE", "VEH_NO", "VehType","VehMode")]

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Assign victim mode for all fatal injuries (non-motorized + motorized)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Person file with all victims, including non-vehicle victims
person <- merge(person, vehicles,  by = c("ST_CASE","VEH_NO"), all.x = TRUE)

# Assign mode type for vehicular fatal injuries
person$VMODE <- ifelse(is.na(person$VMODE), person$VehMode, person$VMODE)
person <- person[order(person$victim_id),c("victim_id", "ST_CASE","PER_NO",
                "VE_FORMS", "VMODE")]

# Assign missing victim mode
person[is.na(person$VMODE),"VMODE"] <- 7

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Merge with Vehicle file to pick up all vehicles involved in crash
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

person_vehicles <- merge(person, vehicles, by = c("ST_CASE"), all.x = TRUE)
person_vehicles <- person_vehicles[,c("victim_id","VE_FORMS","VMODE", "VEH_NO",
                  "VehMode")]

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Create wide file with all parties to a collision on the same row
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Wide file with victim and all parties
victims <- reshape(person_vehicles,
                   timevar = "VEH_NO",
                   idvar = c("victim_id", "VMODE","VE_FORMS"),
                   direction = "wide")
# count of victims by mode
table(victims$VMODE)

# Count of rows with missing data for striking vehicle
apply(victims[,c(4:30)],2,function(x) sum(!is.na(x)))

# VehMode.1  VehMode.2  VehMode.3  VehMode.4  VehMode.5  VehMode.6  VehMode.7  VehMode.8  VehMode.9 VehMode.10 VehMode.11 VehMode.22 
# 35698      16434       3020        854        340        142         68         50         26         19          6          1 
# VehMode.19 VehMode.17 VehMode.24 VehMode.23 VehMode.27 VehMode.20 VehMode.26 VehMode.14 VehMode.15 VehMode.12 VehMode.21 VehMode.16 
# 1          1          1          1          1          1          1          2          1          5          1          1 
# VehMode.25 VehMode.18 VehMode.13 
# 1          1          2 

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Striking Vehicle with 1 party Colisisons is NOV
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# victims$SMODE <- ifelse(victims$VE_FORMS==1, 9, NA)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Striking Vehicle in collision with 2 or more parties, is mode with
# the highest numerical value (mass) that isn't the victim mode, unless
# other parties are the same mode as victim mode
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Striking Vehicle
# Mass 
# Precedence Code
# ~~~~~~~~~~~~~~~~

# Pedestrian  1
# Bicycle     2
# Motorcycle  3
# Car         4
# Bus         5
# Truck       6
# NOV         7
# Missing     8
# ~~~~~~~~~~~~~~~~

# Striking Vehicle with Largest mass
victims$max    <- apply(victims[,c(4:30)], MARGIN=1, FUN=max, na.rm=TRUE)
victims$sumveh <- apply(victims[,c(4:30)], MARGIN=1, function(x) sum(!is.na(x)))
victims$SMODE  <- ifelse(victims$max > victims$VMODE, victims$max, victims$VMODE)

# Striking mode NOV for non-motorized and missing victims
victims$SMODE <- ifelse(victims$VMODE <= 2 & victims$sumveh==0, 7, victims$SMODE)

# Striking Mode NOV for motorized modes
victims$SMODE <- ifelse(victims$VMODE >= 3 & victims$sumveh==1, 7, victims$SMODE)

# Striking Mode of Bus victims of car collisions
victims$SMODE <- ifelse(victims$VMODE == 5 & victims$sumveh == 2 &
                  ((victims$VehMode.1 == 5 & victims$VehMode.2 == 4) | 
                  ( victims$VehMode.1 == 4 & victims$VehMode.2== 5)), 4,
                  victims$SMODE)

# Striking Mode of Truck victims of car collisions
victims$SMODE <- ifelse(victims$VMODE == 6 & victims$sumveh == 2 &
                          ((victims$VehMode.1 == 6 & victims$VehMode.2 == 4) | 
                          ( victims$VehMode.1 == 4 & victims$VehMode.2 == 6)), 4, 
                        victims$SMODE)

# Striking Mode of Truck victims of bus collisions
victims$SMODE <- ifelse(victims$VMODE == 6 & victims$sumveh==2 &
                          ((victims$VehMode.1 == 6 & victims$VehMode.2 == 5) | 
                          (victims$VehMode.1 == 5 & victims$VehMode.2== 6)), 5, 
                        victims$SMODE)

# Assign Striking mode Missing (8) for missing Victim mode (7)
victims$SMODE <- ifelse(victims$VMODE == 7 & victims$sumveh==0, 8, victims$SMODE)

table(victims$VMODE,victims$SMODE, exclude=NULL)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Pick up roadway type from Collision file (ACCIDENT.CSV)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

accident <- read.csv("ACCIDENT2016.csv", header = TRUE, stringsAsFactors = F)  
accident <- accident[,c("ST_CASE","FUNC_SYS")]

# Recode functional system classifiction
# 1 Interstate
# 2 Principal Arterial - Other Freeways and Expressways
# 3 Principal Arterial - Other
# 4 Minor Arterial
# 5 Major Collector
# 6 Minor Collector
# 7 Local
# 96 Trafficway Not in State Inventory
# 98 Not Reported
# 99 Unknown

roadway_groups <- function(x) { 
  if    (is.na(x)) y <- "NA"
  else if (x <= 3)             y <- "Highway"
  else if (x >= 4  & x <= 5)   y <- "Arterial"
  else if (x >= 6  & x <= 7)   y <- "Local"
  else y <- NA
  return(y)
}

# ITHIM Age Sex Groups
accident$Roadway <- sapply(accident$FUNC_SYS,roadway_groups)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Pick up roadway type from Collision file
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

victims$ST_CASE <- floor(as.numeric(victims$victim_id)/100)

victims <- merge(victims, accident, by="ST_CASE", all.x = TRUE)
victims <- victims[,c("victim_id","VMODE","SMODE","Roadway")]

# Totals including missing roadway type
tab_ttl <- as.data.frame.matrix(addmargins(table(victims$VMODE,victims$SMODE)))

#               ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~Striking  Mode ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Victim Mode   Walk(1) Bike(2) Motorcycle(3) Car(4) Bus(5) Truck(6) NOV(7) Missing(8)  Sum(9)
#       Walk(1)
#       Bike(2)
# Motorcycle(3)
#        Car(4)
#        Bus(5)
#      Truck(6)
#    Missing(7)
#       Sum (8)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Create function for imputation stratified by roadway type
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
impute.function <- function(froadtype) {

roadtype <- froadtype
  
victimstemp <- victims[victims$Roadway==roadtype & !is.na(victims$Roadway),]
tabtemp  <- as.data.frame.matrix(addmargins(table(victimstemp$VMODE,victimstemp$SMODE)))

# Add Walk and Bike Striking modes
tabtemp[,c("1","2")] <- 0
tabtemp <- tabtemp[,c("1","2","3","4","5","6","7","8","Sum")]
rti <-  tabtemp

# Impute missing values
newrti <- as.data.frame(matrix(0, ncol = 7, nrow = 6))

# Loop over the ith row and jth column
for(i in 1:6) {
  for(j in 1:7) {
newrti[i,j] = (rti[i,j] +((rti[i,j]/(rti[i,9]-rti[i,8])*rti[i,8])+(rti[i,j]/(rti[8,j]-rti[7,j])*rti[7,j])))*(rti[8,9]/(rti[8,9]-rti[7,8]))
  } # End jth column
}  # End ith row

# Replace all division by 0 with 0s
newrti[is.na(newrti)] <- 0

# Adjust by missing roadway type 
newrti <- newrti*(1/(1 - sum(is.na(victims))/nrow(victims)))

# Add column names
colnames(newrti)[1:7] <- c("walk", "bike", "motorcycle", "car","bus", "truck", "NOV")

newrti$Region <- "United States"
newrti$Severity <- "Fatal"
newrti$Roadway  <- roadtype
newrti$VictimMode <- NA
newrti[1,"VictimMode"] <- "1Walk"
newrti[2,"VictimMode"] <- "2Bike"
newrti[3,"VictimMode"] <- "6Motorcycle"
newrti[4,"VictimMode"] <- "4Car"
newrti[5,"VictimMode"] <- "3Bus"
newrti[6,"VictimMode"] <- "5Truck"

# Reorder striking Mode columns
newrti <- newrti[,c("Region", "Severity", "Roadway", "VictimMode", "walk", 
                    "bike", "bus", "car", "truck", "motorcycle", "NOV")]

# Reorder Victim Mode rows
newrti <- newrti[order(newrti$VictimMode),]

# strip off ordering character
newrti$VictimMode <- substr(newrti$VictimMode,2,11)

return(newrti)
}  

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Impute Missing Data for Each Roadway type
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
hwy_tab  <- impute.function("Highway")
art_tab  <- impute.function("Arterial")
loc_tab  <- impute.function("Local")

# Check sum
sum(hwy_tab[,5:11]) + sum(art_tab[,5:11]) + sum(loc_tab[,5:11])

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Consolidate in a single file 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

fatal_rti <- rbind(loc_tab,art_tab,hwy_tab)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Write out resuls
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

write.csv(fatal_rti, paste0("rti_baseline_fatal",format(Sys.time(),
           "%Y-%m-%d"),".csv"), row.names=FALSE)

